# Périmètre fonctionnel

xbus-http est une librairie go fournissant un net/http/Handler(Func), qui
lui-même offrira une passerelle http/rest vers XBus.

Un binaire standalone est compris dans le périmètre, mais une utilisation
intégrée avec xbus-gui sera un cas d'usage fréquent, si ce n'est le plus
courant.

# Persistence

La passerelle sera state-less -> aucun besoin de stocker des informations en
base de données directement.

Le passerelle démarrera avec un simple fichier de configuration, à la manière
de xbusctl ou xbus-client.

Un redémarrage intempestif de la passerelle à n'importe quel moment aura une
incidence sur les transferts en cours, en les interrompant, mais ceux-ci seront
repris par le bus comme pour un client ordinaire.

# Sécurité

Un soin particulier sera apporté à ce que la sécurité des échanges soit
contrôlée par le serveur XBus lui-même.

Le principe en est le suivant :

- la passerelle a un compte xbus de type 'passerelle' dont elle a le certificat.
- ce type de compte ne donne aucun droit par défaut, mais permet de passer des
  informations d'authentification pour un autre compte dans le CONNECT. Ainsi,
  la passerelle pourra transmettre une clé d'API ou un fingerprint de
  certificat tiers au bus.
- A l'issue du CONNECT, le compte actif de la connexion obtenue est celui dont
  la clé d'API ou le certificat a été passé, et non le compte de la
  passerelle.

Ce mécanisme impose que la passerelle conserve une connexion vers le bus pour
chaque compte utilisant l'utilisant.

En contrepartie, on s'assure que la sécurité est assurée directement par le
bus, ce qui diminue la surface d'exposition pour les failles de sécurité.

A noter que ce méchanisme est partagé avec le serveur http de xbus-gui, et sera
potentiellement fourni par go-xbus.

# API

L'API probable de la passerelle sera la suivante. Il va sans dire qu'elle est
hautement sujette à changement au cours de son implémentation.

## Fragmentation des enveloppes

Http n'ayant pas de limitation de taille de 'message', la fragmentation des
enveloppes n'est pas nécessaire, mais elle reste utile pour éviter des timeout
dans un certain nombre de cas.

La passerelle aura pour responsabilité de convertir une enveloppe entrante trop
grosse en plusieurs fragments d'enveloppe vers le bus, et à l'inverse de
rassembler un flux de fragment enveloppe venant du bus en un seul payload pour
le client final.

## Meta-données

Dans le cadre d'une authentification par clé d'API, les requêtes devront
contenir un entête d'authentication dont la nature reste à déterminer
(json-token, basic-authentication...)


## Emission

`POST /<actor_id>/output/<output_id>`

Envoie une enveloppe en tant que <actor_id> sur la sortie <output_id>. Le
format de l'enveloppe est le même que sur le bus.

## Reception

`POST /<actor_id>/processingend`

Informe le bus de la fin du processing

### pull

`GET /<actor_id>/input/<input_id>[?wait=<delay>]`

Attends une enveloppe entrante et la retourne dès qu'elle est prête. Le temps
d'attente par défaut sera à déterminer (probablement de l'ordre de la minute,
pas plus pour éviter les timeout).

`POST /<actor_id>/input/<input_id>/ack`

Informe le bus de la bonne reception d'un fragment d'envelope
Devra impérativement être appelé immédiatement après la récupération d'un
fragment.

### push

`POST /<actor_id>/webhook`

Associe une url à un acteur. Seront postées sur cette url les enveloppes
(potentiellement fragmentées) destinées à l'acteur.


# WebSocket

Un upgrade des points d'entrées de l'API en WebSocket est envisageable, ce qui
permettra une plus grande réactivité des clients http et une économie
substantielle de ressources.

Les messages échangés sur ces sockets seront strictement les mêmes que les
payload d'entrée et sortie des API correspondants. Il ne faut donc pas les
confondre avec la passerelle nats-websocket intégrée à xbus-gui.

Seule subtilité en réception pull, le `ACK` pourra être renvoyé dans le même
socket au lieu de poster sur une autre url.

# Modifications de Xbus

Quelques modifications de Xbus sont nécessaires pour atteindre le plein
potentiel de la passerelle.

## Requises

- Ajout du type de compte 'passerelle'
- Ajout du champs 'api_key' sur Account
- Implémentation de l'authentification depuis un compte passerelle, basée sur
  un clé d'API ou un fingerprint. Ce dernier cas permettra au client d'utiliser
  certificat xbus dont la validité sera établie par la passerelle.

## Fortement recommandées

- Stockage d'une configuration sur les acteurs, comme elle existe déjà dans le
  fichier de configuration de xbus-client.

  Cela permettra, pour xbus-http, de ne stocker aucune configuration autre que
  son propre certificat de compte.

  L'autre avantage considérable est que l'administrateur Xbus pourra configurer
  les acteurs directement avec l'UI standard de Xbus, et non en devant modifier
  un fichier de configuration. Xbus-client pourra bénéficier de cette
  amélioration.

## Recommandées

- Ajout d'une API pour demander la réémission d'une enveloppe vers un acteur.
  
  Actuellement, la réception d'un ACK de l'acteur est considéré comme une
  réception définitive du fragment. Dans le cas de la passerelle, et si on
  rassemble les fragments en un seul payload, alors la réception d'un ACK
  signifiera simplement que le fragment a été joint à un envoi potentiellement
  plus grand, et qui peut encore échouer.

  Si un tel échec survient, le client final ne recevra jamais l'enveloppe ou
  certains fragment.

  Il faut donc pouvoir demander la réémission d'une enveloppe, complète ou
  depuis une position antérieure, y compris si un ACK a déjà été envoyé pour le
  dernier fragment (mais toujours avant le processingend).

  Sans cette modification, le client recevra nécessairement des enveloppes
  fragmentées (sauf les petites évidemment).
